#ifndef _DESCRIPTORS_H_
#define _DESCRIPTORS_H_

	/* Includes: */
		#include <avr/pgmspace.h>
		#include <LUFA/Drivers/USB/USB.h>

	/* Type Defines: */
		/** Type define for the device configuration descriptor structure. This must be defined in the
		 *  application code, as the configuration descriptor contains several sub-descriptors which
		 *  vary between devices, and which describe the device's usage to the host.
		 */
		typedef struct
		{
			USB_Descriptor_Configuration_Header_t Config;

			// PAD Interface
			USB_Descriptor_Interface_t            PAD_Interface_LED;
			USB_Descriptor_Endpoint_t             PAD_OUT_LED1;
			USB_Descriptor_Endpoint_t             PAD_OUT_LED2;
			USB_Descriptor_Interface_t            PAD_Interface_BUTTON;
			USB_Descriptor_Endpoint_t             PAD_IN_BUTTON1;
			USB_Descriptor_Endpoint_t             PAD_IN_BUTTON2;
		} USB_Descriptor_Configuration_t;

		/** Enum for the device interface descriptor IDs within the device. Each interface descriptor
		 *  should have a unique ID index associated with it, which can be used to refer to the
		 *  interface from other descriptors.
		 */
		enum InterfaceDescriptors_t
		{
			INTERFACE_ID_LED = 0,
			INTERFACE_ID_BUTTON = 1,
		};

		/** Enum for the device string descriptor IDs within the device. Each string descriptor should
		 *  have a unique ID index associated with it, which can be used to refer to the string from
		 *  other descriptors.
		 */
		enum StringDescriptors_t
		{
			STRING_ID_Language     = 0, /**< Supported Languages string descriptor ID (must be zero) */
			STRING_ID_Manufacturer = 1, /**< Manufacturer string ID */
			STRING_ID_Product      = 2, /**< Product string ID */
			STRING_ID_Serial       = 3, /**< Serial number string ID */
		};


	/* Macros: */
		/** Endpoint address of the Keyboard HID reporting OUT endpoint. */
		#define PAD_OUT_EP_LED1 (ENDPOINT_DIR_OUT | 1)
		#define PAD_OUT_EP_LED2 (ENDPOINT_DIR_OUT | 2)

		/** Endpoint address of the Keyboard HID reporting IN endpoint. */
		#define PAD_IN_EP_BUTTON1 (ENDPOINT_DIR_IN | 3)
		#define PAD_IN_EP_BUTTON2 (ENDPOINT_DIR_IN | 4)

		/** Size in bytes of the Keyboard HID reporting IN and OUT endpoints. */
		#define PAD_EPSIZE 1

	/* Function Prototypes: */
		uint16_t CALLBACK_USB_GetDescriptor(const uint16_t wValue,
		                                    const uint16_t wIndex,
		                                    const void** const DescriptorAddress)
		                                    ATTR_WARN_UNUSED_RESULT ATTR_NON_NULL_PTR_ARG(3);
#endif

