#include <libusb-1.0/libusb.h>
#include <stdio.h>
#include <stdlib.h>

// arduino ID vendor and product
#define ID_VENDOR 0x2341
#define ID_PRODUCT 0x01

// gcc file.c -l usb-1.0

int main(){
	// init
	libusb_context *context;
	int status = libusb_init(&context);
	if(status != 0){perror("libusb_init"); exit(-1);}

	// devices list
	libusb_device **devices;
	ssize_t devices_count = libusb_get_device_list(context, &devices);
	if(devices_count < 0){perror("libusb_get_device_list"); exit(-1);}
	for(int i=0; i<devices_count; i++){
		libusb_device *device = devices[i];

		// device description
		struct libusb_device_descriptor desc;
		int status = libusb_get_device_descriptor(device, &desc);
		if(status != 0) continue;
		
		// search for device
		if(desc.idVendor != ID_VENDOR
				|| desc.idProduct != ID_PRODUCT)
			continue;

		// open connection
		libusb_device_handle *handle;
		status = libusb_open(device, &handle);
		if(status != 0){ perror("libusb_open"); exit(-1); }
		
		// prepare config
		struct libusb_config_descriptor *config_desc;
		status = libusb_get_config_descriptor(device, 0, &config_desc);
		if(status != 0){ perror("libusb_get_config_descriptor"); exit(-1); }
		int configuration = config_desc->bConfigurationValue;

		// detach kernel
		for(int j=0; j<config_desc->bNumInterfaces; j++){
			int interface = config_desc->interface[j].altsetting[0].bInterfaceNumber;
			if(libusb_kernel_driver_active(handle, interface)){
				status = libusb_detach_kernel_driver(handle, interface);
				if(status != 0){ perror("libusb_detach_kernel_driver"); exit(-1); }
			}
		}
		
		// use config
		status = libusb_set_configuration(handle, configuration);
		if(status != 0){ perror("libusb_set_configuration"); exit(-1); }
		
		// claim interfaces
		for(int j=0; j<config_desc->bNumInterfaces; j++){
			struct libusb_interface_descriptor interface_desc = config_desc->interface[j].altsetting[0];
			
			int interface = interface_desc.bInterfaceNumber;
			status = libusb_claim_interface(handle, interface);
			if(status != 0){ perror("libusb_claim_interface"); exit(-1); }
			
			// find endpoints
			for(int k=0; k<interface_desc.bNumEndpoints; k++){
				struct libusb_endpoint_descriptor endpoint_desc = interface_desc.endpoint[k];
				
				// list endpoints bEndpointAddress and bmAttributes
				printf("Endpoint inter. %d, num. %d\n", j, k);
				printf("bEndpointAddress = 0x%x\n", endpoint_desc.bEndpointAddress);
				printf("bmAttributes = 0x%x\n", endpoint_desc.bmAttributes);
			}
			
			// release interfaces
			status = libusb_release_interface(handle, interface);
			if(status != 0){ perror("libusb_release_interface"); exit(-1); }
		}
		
		// free config
		libusb_free_config_descriptor(config_desc);

		// close connection
		libusb_close(handle);
	}
	libusb_free_device_list(devices, 1);

	libusb_exit(context);
}
